#include "Draw.h"
Draw* Draw::instance = 0;

Draw* Draw::getInstance(){
	if(!instance){
		instance = new Draw();
	}
	return instance;
}

Draw::Draw(){
	drawColor = 0xFFFFFFFF;
	drawFont = GRRLIB_LoadTexture(f8r); //Do not change this once its loaded, GRRLIB doesnt like that
}
Draw::~Draw(){}

void Draw::setColor(u32 color)							{ drawColor = color; }
void Draw::drawText(int line, string text)				{ drawFreeText(10,14*line,text,1); }
void Draw::drawFreeText(int x, int y,string text,int size){
	char tmp[1024]; //1024 = max in GRRLIB
	if(text.size() > 1024){
		strcpy(tmp,"String too large"); //Otherwise, this would be buffer overflow-prone
	} 
	else{
		strcpy(tmp,text.c_str()); //c_str returns a const, we need a normal char*
	}
	GRRLIB_Printf(x,y,drawFont,drawColor,size,tmp);
}

void Draw::drawPixel(int x, int y)						{ GRRLIB_Plot		(x,y,drawColor);			}
void Draw::drawLine(int x1, int y1, int x2, int y2)		{ GRRLIB_Line		(x1,y1,x2,y2,drawColor);	}
void Draw::drawBox(int x1, int y1, int w, int h)		{ GRRLIB_Rectangle	(x1,y1,w,h,drawColor,0); 	}
void Draw::fillBox(int x1, int y1, int w, int h)		{ GRRLIB_Rectangle	(x1,y1,w,h,drawColor,1); 	}

void Draw::fill50Box(int x, int y, int w, int h){
	u32 colorNow = drawColor;
	
	drawBox(x,y,w,h);
	setColor(drawColor - 0x80000000);
	fillBox(x+1,y+1,w-2,h-2);
	
	setColor(colorNow);
}
	
void Draw::drawSliderBox(int x, int y, int w, int h, int sliderpos){
			u32 colorNow = drawColor;
			//Draw marker boxes
			setColor(C_GREEN);
			drawBox(x,y,w,h);
			
			//Draw center lines
			setColor(C_WHITE);
			drawLine(x+w/2,y,x+w/2,y+h); //x

			//Draw pos boxes
			setColor(C_RED);
			if(sliderpos > w){ sliderpos = w; }
			if(sliderpos < 0){ sliderpos = 0; }
			drawBox(x-2+sliderpos,y,5,h);
			drawBox(x-1+sliderpos,y,3,h);
			
			setColor(colorNow);
}

void Draw::fillScreen()									{ GRRLIB_FillScreen	(drawColor); 				}

u8* Draw::getTextureFromBuffer(const unsigned char* buffer){ return GRRLIB_LoadTexture(buffer); };
void Draw::drawTexture(int x, int y, int w, int h, u8* texture, bool frame, float rotation, float scalex, float scaley, float alpha){
	if( (w%4) != 0 ){ return; }
	if( (h%4) != 0 ){ return; }
	GRRLIB_DrawImg(x,y,w,h,texture,rotation,scalex,scaley,alpha);
	if(frame){
		drawBox(x-1,y-1,w+1,h+1);
	}
};

void Draw::render(){ GRRLIB_Render(); }

//String conversions for drawText
string Draw::intToString(int integer){
	ostringstream bufstream;
	bufstream << integer;
	string buf = bufstream.str();
	return buf;
}

string Draw::doubleToString(double doubleval){
	ostringstream bufstream;
	bufstream << doubleval;
	string buf = bufstream.str();
	return buf;
}

string Draw::boolToString(bool boolean){
	if(boolean){ return "true"; }
	return "false";
}
